//!
//!
use crate::repositories::{
  bank::models::Bank as _Bank, member::schema::MemberState as _MemberState,
  member_class::models::MemberClass as _MemberClass,
  member_game_setting::models::MemberGameSetting as _MemberGameSetting,
  member_level::models::MemberLevel as _MemberLevel,
  member_settlement_setting::models::MemberSettlementSetting as _MemberSettlementSetting,
  site::models::Site as _Site,
};
use diesel::deserialize::QueryableByName;

///
#[derive(PartialEq, PartialOrd, Debug, Clone)]
pub struct ParentMemberModel {
  ///
  pub id: uuid::Uuid,
  ///
  pub site: _Site,
  ///
  pub member_class: _MemberClass,
  ///
  pub member_level: _MemberLevel,
  ///
  pub bank_account: Option<MemberBankAccountModel>,
  ///
  pub username: String,
  ///
  pub password: String,
  ///
  pub nickname: String,
  ///
  pub parent_member_id: Option<uuid::Uuid>,
  ///
  pub child_member_count: i64,
  ///
  pub mobile_phone_number: Option<String>,
  ///
  pub state: _MemberState,
  ///
  pub state_changed_at: Option<i64>,
  ///
  pub member_game_setting: Option<_MemberGameSetting>,
  ///
  pub member_settlement_setting: Option<_MemberSettlementSetting>,
  ///
  pub last_signined_ip: Option<String>,
  ///
  pub last_signined_at: Option<i64>,
  ///
  pub created_at: i64,
  ///
  pub updated_at: i64,
  ///
  pub deleted_at: Option<i64>,
}

///
#[derive(PartialEq, PartialOrd, Debug, Clone)]
pub struct MemberModel {
  ///
  pub id: uuid::Uuid,
  ///
  pub site: _Site,
  ///
  pub member_class: _MemberClass,
  ///
  pub member_level: _MemberLevel,
  ///
  pub bank_account: Option<MemberBankAccountModel>,
  ///
  pub username: String,
  ///
  pub password: String,
  ///
  pub nickname: String,
  ///
  pub parent_member: Option<ParentMemberModel>,
  ///
  pub child_member_count: i64,
  ///
  pub mobile_phone_number: Option<String>,
  ///
  pub state: _MemberState,
  ///
  pub state_changed_at: Option<i64>,
  ///
  pub member_game_setting: Option<_MemberGameSetting>,
  ///
  pub member_settlement_setting: Option<_MemberSettlementSetting>,
  ///
  pub last_signined_ip: Option<String>,
  ///
  pub last_signined_at: Option<i64>,
  ///
  pub created_at: i64,
  ///
  pub updated_at: i64,
  ///
  pub deleted_at: Option<i64>,
}

impl QueryableByName<diesel::pg::Pg> for MemberModel {
  fn build<R: diesel::row::NamedRow<diesel::pg::Pg>>(row: &R) -> diesel::deserialize::Result<Self> {
    let parent_member = match row
      .get::<diesel::sql_types::Nullable<diesel::sql_types::Uuid>, Option<uuid::Uuid>>(
        "m_parent_member_id",
      )? {
      Some(_) => {
        let site = _Site {
          id: row.get("pm_s_id")?,
          url: row.get("pm_s_url")?,
          name: row.get("pm_s_name")?,
          path: row.get("pm_s_path")?,
          show: row.get("pm_s_show")?,
          can_use: row.get("pm_s_can_use")?,
          memo: row.get("pm_s_memo")?,
          expires_at: row.get("pm_s_expires_at")?,
          created_at: row.get("pm_s_created_at")?,
          updated_at: row.get("pm_s_updated_at")?,
        };

        let member_class = _MemberClass {
          id: row.get("pm_mc_id")?,
          parent_id: row.get("pm_mc_parent_id")?,
          name: row.get("pm_mc_name")?,
          show: row.get("pm_mc_show")?,
          created_at: row.get("pm_mc_created_at")?,
          updated_at: row.get("pm_mc_updated_at")?,
        };

        let member_level = _MemberLevel {
          id: row.get("pm_ml_id")?,
          name: row.get("pm_ml_name")?,
          show: row.get("pm_ml_show")?,
          sort_order: row.get("pm_ml_sort_order")?,
          created_at: row.get("pm_ml_created_at")?,
          updated_at: row.get("pm_ml_updated_at")?,
        };

        let member_game_setting = match row
          .get::<diesel::sql_types::Nullable<diesel::sql_types::Uuid>, Option<uuid::Uuid>>(
            "pm_mgs_id",
          )? {
          Some(_) => Some(_MemberGameSetting {
            id: row.get("pm_mgs_id")?,
            member_id: row.get("pm_mgs_member_id")?,
            can_bet_casino: row.get("pm_mgs_can_bet_casino")?,
            can_bet_slot: row.get("pm_mgs_can_bet_slot")?,
            can_bet_powerball: row.get("pm_mgs_can_bet_powerball")?,
            can_bet_powerladder: row.get("pm_mgs_can_bet_powerladder")?,
            can_bet_eos: row.get("pm_mgs_can_bet_eos")?,
            can_bet_bogglepowerball: row.get("pm_mgs_can_bet_bogglepowerball")?,
            created_at: row.get("pm_mgs_created_at")?,
            updated_at: row.get("pm_mgs_updated_at")?,
          }),
          None => None,
        };

        let member_settlement_setting = match row.get::<diesel::sql_types::Nullable<
          diesel::sql_types::Uuid,
        >, Option<uuid::Uuid>>("pm_mss_id")?
        {
          Some(_) => Some(_MemberSettlementSetting {
            id: row.get("pm_mss_id")?,
            member_id: row.get("pm_mss_member_id")?,
            can_exchange: row.get("pm_mss_can_exchange")?,
            can_first_charge_comp: row.get("pm_mss_can_first_charge_comp")?,
            can_per_charge_comp: row.get("pm_mss_can_per_charge_comp")?,
            manual_payment_type_for_partner: row.get("pm_mss_manual_payment_type_for_partner")?,
            settlement_type: row.get("pm_mss_settlement_type")?,
            rate_casino: row.get("pm_mss_rate_casino")?,
            rate_casino_loosing: row.get("pm_mss_rate_casino_loosing")?,
            rate_casino_bacara: row.get("pm_mss_rate_casino_bacara")?,
            rate_casino_roulette: row.get("pm_mss_rate_casino_roulette")?,
            rate_casino_dragon_tiger: row.get("pm_mss_rate_casino_dragon_tiger")?,
            rate_slot: row.get("pm_mss_rate_slot")?,
            rate_slot_loosing: row.get("pm_mss_rate_slot_loosing")?,
            rate_powerball_single: row.get("pm_mss_rate_powerball_single")?,
            rate_powerball_combo: row.get("pm_mss_rate_powerball_combo")?,
            rate_powerladder_single: row.get("pm_mss_rate_powerladder_single")?,
            rate_powerladder_combo: row.get("pm_mss_rate_powerladder_combo")?,
            rate_eos_single: row.get("pm_mss_rate_eos_single")?,
            rate_eos_combo: row.get("pm_mss_rate_eos_combo")?,
            rate_bogglepowerball_single: row.get("pm_mss_rate_bogglepowerball_single")?,
            rate_bogglepowerball_combo: row.get("pm_mss_rate_bogglepowerball_combo")?,
            created_at: row.get("pm_mss_created_at")?,
            updated_at: row.get("pm_mss_updated_at")?,
          }),
          None => None,
        };

        let member_bank_account = match row
          .get::<diesel::sql_types::Nullable<diesel::sql_types::Uuid>, Option<uuid::Uuid>>(
            "pm_mba_id",
          )? {
          Some(_) => {
            let bank = _Bank {
              id: row.get("pm_mba_b_id")?,
              name: row.get("pm_mba_b_name")?,
              sort_order: row.get("pm_mba_b_sort_order")?,
              show: row.get("pm_mba_b_show")?,
              can_use: row.get("pm_mba_b_can_use")?,
              memo: row.get("pm_mba_b_memo")?,
              created_at: row.get("pm_mba_b_created_at")?,
              updated_at: row.get("pm_mba_b_updated_at")?,
            };

            Some(MemberBankAccountModel {
              id: row.get("pm_mba_id")?,
              member_id: row.get("pm_mba_member_id")?,
              bank,
              name: row.get("pm_mba_name")?,
              account_number: row.get("pm_mba_account_number")?,
              exchange_password: row.get("pm_mba_exchange_password")?,
              memo: row.get("pm_mba_memo")?,
              created_at: row.get("pm_mba_created_at")?,
              updated_at: row.get("pm_mba_updated_at")?,
            })
          }
          None => None,
        };

        Some(ParentMemberModel {
          id: row.get("pm_id")?,
          site,
          member_class,
          member_level,
          username: row.get("pm_username")?,
          password: row.get("pm_password")?,
          nickname: row.get("pm_nickname")?,
          mobile_phone_number: row.get("pm_mobile_phone_number")?,
          state: row.get("pm_state")?,
          state_changed_at: row.get("pm_state_changed_at")?,
          parent_member_id: row.get("pm_parent_member_id")?,
          child_member_count: row.get("pm_child_member_count")?,
          member_game_setting,
          member_settlement_setting,
          bank_account: member_bank_account,
          last_signined_ip: row.get("pm_last_signined_ip")?,
          last_signined_at: row.get("pm_last_signined_at")?,
          created_at: row.get("pm_created_at")?,
          updated_at: row.get("pm_updated_at")?,
          deleted_at: row.get("pm_deleted_at")?,
        })
      }
      None => None,
    };

    let site = _Site {
      id: row.get("m_s_id")?,
      url: row.get("m_s_url")?,
      name: row.get("m_s_name")?,
      path: row.get("m_s_path")?,
      show: row.get("m_s_show")?,
      can_use: row.get("m_s_can_use")?,
      memo: row.get("m_s_memo")?,
      expires_at: row.get("m_s_expires_at")?,
      created_at: row.get("m_s_created_at")?,
      updated_at: row.get("m_s_updated_at")?,
    };

    let member_class = _MemberClass {
      id: row.get("m_mc_id")?,
      parent_id: row.get("m_mc_parent_id")?,
      name: row.get("m_mc_name")?,
      show: row.get("m_mc_show")?,
      created_at: row.get("m_mc_created_at")?,
      updated_at: row.get("m_mc_updated_at")?,
    };

    let member_level = _MemberLevel {
      id: row.get("m_ml_id")?,
      name: row.get("m_ml_name")?,
      show: row.get("m_ml_show")?,
      sort_order: row.get("m_ml_sort_order")?,
      created_at: row.get("m_ml_created_at")?,
      updated_at: row.get("m_ml_updated_at")?,
    };

    let member_game_setting = match row
      .get::<diesel::sql_types::Nullable<diesel::sql_types::Uuid>, Option<uuid::Uuid>>("m_mgs_id")?
    {
      Some(_) => Some(_MemberGameSetting {
        id: row.get("m_mgs_id")?,
        member_id: row.get("m_mgs_member_id")?,
        can_bet_casino: row.get("m_mgs_can_bet_casino")?,
        can_bet_slot: row.get("m_mgs_can_bet_slot")?,
        can_bet_powerball: row.get("m_mgs_can_bet_powerball")?,
        can_bet_powerladder: row.get("m_mgs_can_bet_powerladder")?,
        can_bet_eos: row.get("m_mgs_can_bet_eos")?,
        can_bet_bogglepowerball: row.get("m_mgs_can_bet_bogglepowerball")?,
        created_at: row.get("m_mgs_created_at")?,
        updated_at: row.get("m_mgs_updated_at")?,
      }),
      None => None,
    };

    let member_settlement_setting = match row
      .get::<diesel::sql_types::Nullable<diesel::sql_types::Uuid>, Option<uuid::Uuid>>("m_mss_id")?
    {
      Some(_) => Some(_MemberSettlementSetting {
        id: row.get("m_mss_id")?,
        member_id: row.get("m_mss_member_id")?,
        can_exchange: row.get("m_mss_can_exchange")?,
        can_first_charge_comp: row.get("m_mss_can_first_charge_comp")?,
        can_per_charge_comp: row.get("m_mss_can_per_charge_comp")?,
        manual_payment_type_for_partner: row.get("m_mss_manual_payment_type_for_partner")?,
        settlement_type: row.get("m_mss_settlement_type")?,
        rate_casino: row.get("m_mss_rate_casino")?,
        rate_casino_loosing: row.get("m_mss_rate_casino_loosing")?,
        rate_casino_bacara: row.get("m_mss_rate_casino_bacara")?,
        rate_casino_roulette: row.get("m_mss_rate_casino_roulette")?,
        rate_casino_dragon_tiger: row.get("m_mss_rate_casino_dragon_tiger")?,
        rate_slot: row.get("m_mss_rate_slot")?,
        rate_slot_loosing: row.get("m_mss_rate_slot_loosing")?,
        rate_powerball_single: row.get("m_mss_rate_powerball_single")?,
        rate_powerball_combo: row.get("m_mss_rate_powerball_combo")?,
        rate_powerladder_single: row.get("m_mss_rate_powerladder_single")?,
        rate_powerladder_combo: row.get("m_mss_rate_powerladder_combo")?,
        rate_eos_single: row.get("m_mss_rate_eos_single")?,
        rate_eos_combo: row.get("m_mss_rate_eos_combo")?,
        rate_bogglepowerball_single: row.get("m_mss_rate_bogglepowerball_single")?,
        rate_bogglepowerball_combo: row.get("m_mss_rate_bogglepowerball_combo")?,
        created_at: row.get("m_mss_created_at")?,
        updated_at: row.get("m_mss_updated_at")?,
      }),
      None => None,
    };

    let member_bank_account = match row
      .get::<diesel::sql_types::Nullable<diesel::sql_types::Uuid>, Option<uuid::Uuid>>("m_mba_id")?
    {
      Some(_) => {
        let bank = _Bank {
          id: row.get("m_mba_b_id")?,
          name: row.get("m_mba_b_name")?,
          sort_order: row.get("m_mba_b_sort_order")?,
          show: row.get("m_mba_b_show")?,
          can_use: row.get("m_mba_b_can_use")?,
          memo: row.get("m_mba_b_memo")?,
          created_at: row.get("m_mba_b_created_at")?,
          updated_at: row.get("m_mba_b_updated_at")?,
        };

        Some(MemberBankAccountModel {
          id: row.get("m_mba_id")?,
          member_id: row.get("m_mba_member_id")?,
          bank,
          name: row.get("m_mba_name")?,
          account_number: row.get("m_mba_account_number")?,
          exchange_password: row.get("m_mba_exchange_password")?,
          memo: row.get("m_mba_memo")?,
          created_at: row.get("m_mba_created_at")?,
          updated_at: row.get("m_mba_updated_at")?,
        })
      }
      None => None,
    };

    Ok(MemberModel {
      id: row.get("m_id")?,
      site,
      member_class,
      member_level,
      username: row.get("m_username")?,
      password: row.get("m_password")?,
      nickname: row.get("m_nickname")?,
      mobile_phone_number: row.get("m_mobile_phone_number")?,
      state: row.get("m_state")?,
      state_changed_at: row.get("m_state_changed_at")?,
      member_game_setting,
      member_settlement_setting,
      bank_account: member_bank_account,
      parent_member,
      child_member_count: row.get("m_child_member_count")?,
      last_signined_ip: row.get("m_last_signined_ip")?,
      last_signined_at: row.get("m_last_signined_at")?,
      created_at: row.get("m_created_at")?,
      updated_at: row.get("m_updated_at")?,
      deleted_at: row.get("m_deleted_at")?,
    })
  }
}

#[derive(Eq, Hash, PartialEq, PartialOrd, Debug, Clone)]
pub struct MemberBankAccountModel {
  ///
  pub id: uuid::Uuid,
  ///
  pub member_id: uuid::Uuid,
  ///
  pub bank: _Bank,
  ///
  pub name: String,
  ///
  pub account_number: String,
  ///
  pub exchange_password: String,
  ///
  pub memo: Option<String>,
  ///
  pub created_at: i64,
  ///
  pub updated_at: i64,
}
